@@ -30,7 +30,7 @@
 #include <QPair>
 #include <QVector>
 
-#include "qgis.h"
+#include "qgis_sip.h"
 #include "qgsmaplayer.h"
 #include "qgsraster.h"
 #include "qgsrasterdataprovider.h"
@@ -43,7 +43,6 @@ class QgsMapToPixel;
 class QgsRasterRenderer;
 class QgsRectangle;
 class QImage;
-class QLibrary;
 class QPixmap;
 class QSlider;
 
@@ -57,20 +56,20 @@ typedef QList < QPair< QString, QColor > > QgsLegendColorList;
  *  The qgsrasterlayer class makes use of gdal for data io, and thus supports
  *  any gdal supported format. The constructor attempts to infer what type of
  *  file (LayerType) is being opened - not in terms of the file format (tif, ascii grid etc.)
- *  but rather in terms of whether the image is a GRAYSCALE, PaletteD or Multiband,
+ *  but rather in terms of whether the image is a GRAYSCALE, PaletteD or Multiband.
  *
  *  Within the three allowable raster layer types, there are 8 permutations of
  *  how a layer can actually be rendered. These are defined in the DrawingStyle enum
  *  and consist of:
  *
- *  SingleBandGray -> a GRAYSCALE layer drawn as a range of gray colors (0-255)
- *  SingleBandPseudoColor -> a GRAYSCALE layer drawn using a pseudocolor algorithm
- *  PalettedSingleBandGray -> a PaletteD layer drawn in gray scale (using only one of the color components)
- *  PalettedSingleBandPseudoColor -> a PaletteD layer having only one of its color components rendered as pseudo color
- *  PalettedMultiBandColor -> a PaletteD image where the bands contains 24bit color info and 8 bits is pulled out per color
- *  MultiBandSingleBandGray -> a layer containing 2 or more bands, but using only one band to produce a grayscale image
- *  MultiBandSingleBandPseudoColor -> a layer containing 2 or more bands, but using only one band to produce a pseudocolor image
- *  MultiBandColor -> a layer containing 2 or more bands, mapped to the three RGBcolors. In the case of a multiband with only two bands, one band will have to be mapped to more than one color
+ *  - SingleBandGray -> a GRAYSCALE layer drawn as a range of gray colors (0-255)
+ *  - SingleBandPseudoColor -> a GRAYSCALE layer drawn using a pseudocolor algorithm
+ *  - PalettedSingleBandGray -> a PaletteD layer drawn in gray scale (using only one of the color components)
+ *  - PalettedSingleBandPseudoColor -> a PaletteD layer having only one of its color components rendered as pseudo color
+ *  - PalettedMultiBandColor -> a PaletteD image where the bands contains 24bit color info and 8 bits is pulled out per color
+ *  - MultiBandSingleBandGray -> a layer containing 2 or more bands, but using only one band to produce a grayscale image
+ *  - MultiBandSingleBandPseudoColor -> a layer containing 2 or more bands, but using only one band to produce a pseudocolor image
+ *  - MultiBandColor -> a layer containing 2 or more bands, mapped to the three RGBcolors. In the case of a multiband with only two bands, one band will have to be mapped to more than one color
  *
  *  Each of the above mentioned drawing styles is implemented in its own draw* function.
  *  Some of the drawing styles listed above require statistics about the layer such
@@ -97,7 +96,7 @@ typedef QList < QPair< QString, QColor > > QgsLegendColorList;
  *     QgsRasterLayer *myRasterLayer = new QgsRasterLayer(myFileNameQString, myBaseNameQString);
  * \endcode
  *
- *  In order to automate redrawing of a raster layer, you should like it to a map canvas like this :
+ *  In order to automate redrawing of a raster layer, you should link it to a map canvas like this:
  *
  * \code{.cpp}
  *     QObject::connect( myRasterLayer, SIGNAL(repaintRequested()), mapCanvas, SLOT(refresh()) );
@@ -174,12 +173,36 @@ class CORE_EXPORT QgsRasterLayer : public QgsMapLayer
       /**
        * Constructor for LayerOptions.
        */
-      explicit LayerOptions( bool loadDefaultStyle = true )
+      explicit LayerOptions( bool loadDefaultStyle = true,
+                             const QgsCoordinateTransformContext &transformContext = QgsCoordinateTransformContext() )
         : loadDefaultStyle( loadDefaultStyle )
+        , transformContext( transformContext )
       {}
 
-      //! Sets to true if the default layer style should be loaded
+      //! Sets to TRUE if the default layer style should be loaded
       bool loadDefaultStyle = true;
+
+      /**
+       * Coordinate transform context
+       * \since QGIS 3.8
+       */
+      QgsCoordinateTransformContext transformContext = QgsCoordinateTransformContext();
+
+      /**
+       * Controls whether the layer is allowed to have an invalid/unknown CRS.
+       *
+       * If TRUE, then no validation will be performed on the layer's CRS and the layer
+       * layer's crs() may be invalid() (i.e. the layer will have no georeferencing available
+       * and will be treated as having purely numerical coordinates).
+       *
+       * If FALSE (the default), the layer's CRS will be validated using QgsCoordinateReferenceSystem::validate(),
+       * which may cause a blocking, user-facing dialog asking users to manually select the correct CRS for the
+       * layer.
+       *
+       * \since QGIS 3.10
+       */
+      bool skipCrsValidation = false;
+
     };
 
     /**
@@ -187,19 +210,19 @@ class CORE_EXPORT QgsRasterLayer : public QgsMapLayer
      *
      * The main tasks carried out by the constructor are:
      *
-     * -Load the rasters default style (.qml) file if it exists
+     * - Load the rasters default style (.qml) file if it exists
      *
-     * -Populate the RasterStatsVector with initial values for each band.
+     * - Populate the RasterStatsVector with initial values for each band.
      *
-     * -Calculate the layer extents
+     * - Calculate the layer extents
      *
-     * -Determine whether the layer is gray, paletted or multiband.
+     * - Determine whether the layer is gray, paletted or multiband.
      *
-     * -Assign sensible defaults for the red, green, blue and gray bands.
+     * - Assign sensible defaults for the red, green, blue and gray bands.
      * */
     explicit QgsRasterLayer( const QString &uri,
                              const QString &baseName = QString(),
-                             const QString &providerKey = "gdal",
+                             const QString &providerType = "gdal",
                              const QgsRasterLayer::LayerOptions &options = QgsRasterLayer::LayerOptions() );
 
     ~QgsRasterLayer() override;
@@ -259,19 +282,60 @@ class CORE_EXPORT QgsRasterLayer : public QgsMapLayer
      */
     void setDataProvider( const QString &provider, const QgsDataProvider::ProviderOptions &options );
 
+    /**
+     * Updates the data source of the layer. The layer's renderer and legend will be preserved only
+     * if the geometry type of the new data source matches the current geometry type of the layer.
+     * \param dataSource new layer data source
+     * \param baseName base name of the layer
+     * \param provider provider string
+     * \param options provider options
+     * \param loadDefaultStyleFlag set to TRUE to reset the layer's style to the default for the
+     * data source
+     * \see dataSourceChanged()
+     * \since QGIS 3.6
+     */
+    void setDataSource( const QString &dataSource, const QString &baseName, const QString &provider, const QgsDataProvider::ProviderOptions &options, bool loadDefaultStyleFlag = false ) override;
+
     /**
      * Returns the raster layer type (which is a read only property).
      */
     LayerType rasterType() { return mRasterType; }
 
-    //! Sets raster renderer. Takes ownership of the renderer object
+    /**
+     * Sets the raster's \a renderer. Takes ownership of the renderer object.
+     * \see renderer()
+     */
     void setRenderer( QgsRasterRenderer *renderer SIP_TRANSFER );
+
+    /**
+     * Returns the raster's renderer.
+     *
+     * \see setRenderer()
+     */
     QgsRasterRenderer *renderer() const { return mPipe.renderer(); }
 
-    //! Sets raster resample filter. Takes ownership of the resample filter object
+    /**
+     * Returns the raster's resample filter.
+     *
+     * \see brightnessFilter()
+     * \see hueSaturationFilter()
+     */
     QgsRasterResampleFilter *resampleFilter() const { return mPipe.resampleFilter(); }
 
+    /**
+     * Returns the raster's brightness/contrast filter.
+     *
+     * \see resampleFilter()
+     * \see hueSaturationFilter()
+     */
     QgsBrightnessContrastFilter *brightnessFilter() const { return mPipe.brightnessFilter(); }
+
+    /**
+     * Returns the raster's hue/saturation filter.
+     *
+     * \see resampleFilter()
+     * \see brightnessFilter()
+     */
     QgsHueSaturationFilter *hueSaturationFilter() const { return mPipe.hueSaturationFilter(); }
 
     /**
@@ -301,10 +365,18 @@ class CORE_EXPORT QgsRasterLayer : public QgsMapLayer
      */
     QString bandName( int bandNoInt ) const;
 
-    // Returns nullptr if not using the data provider model (i.e. directly using GDAL)
+    /**
+     * Returns the source data provider.
+     *
+     * This will be NULLPTR if the layer is invalid.
+     */
     QgsRasterDataProvider *dataProvider() override;
 
-    // Returns nullptr if not using the data provider model (i.e. directly using GDAL)
+    /**
+     * Returns the source data provider.
+     *
+     * This will be NULLPTR if the layer is invalid.
+     */
     const QgsRasterDataProvider *dataProvider() const SIP_PYNAME( constDataProvider ) override;
 
     void reload() override;
@@ -399,10 +471,39 @@ class CORE_EXPORT QgsRasterLayer : public QgsMapLayer
     void setLayerOrder( const QStringList &layers ) override;
     void setSubLayerVisibility( const QString &name, bool vis ) override;
     QDateTime timestamp() const override;
+    bool accept( QgsStyleEntityVisitorInterface *visitor ) const override;
+
+    /**
+     * Writes the symbology of the layer into the document provided in SLD 1.0.0 format
+     * \param node the node that will have the style element added to it.
+     * \param doc the document that will have the QDomNode added.
+     * \param errorMessage reference to string that will be updated with any error messages
+     * \param props a open ended set of properties that can drive/inform the SLD encoding
+     * \returns TRUE in case of success
+     * \since QGIS 3.6
+     */
+    bool writeSld( QDomNode &node, QDomDocument &doc, QString &errorMessage, const QgsStringMap &props = QgsStringMap() ) const;
+
+    /**
+     * If the ignoreExtent flag is set, the layer will also render outside the
+     * bounding box reported by the data provider.
+     * To be used for example for WMS layers with labels or symbology that happens
+     * to be drawn outside the data extent.
+     *
+     * \since QGIS 3.10
+     */
+    bool ignoreExtents() const;
 
   public slots:
     void showStatusMessage( const QString &message );
 
+    /**
+     * Sets the coordinate transform context to \a transformContext
+     *
+     * \since QGIS 3.8
+     */
+    virtual void setTransformContext( const QgsCoordinateTransformContext &transformContext ) override;
+
   protected:
     bool readSymbology( const QDomNode &node, QString &errorMessage, QgsReadWriteContext &context, QgsMapLayer::StyleCategories categories = QgsMapLayer::AllStyleCategories ) override;
     bool readStyle( const QDomNode &node, QString &errorMessage, QgsReadWriteContext &context, QgsMapLayer::StyleCategories categories = QgsMapLayer::AllStyleCategories ) override;
@@ -453,15 +554,15 @@ class CORE_EXPORT QgsRasterLayer : public QgsMapLayer
 
     QgsRasterViewPort mLastViewPort;
 
-    //! [ data provider interface ] Data provider key
-    QString mProviderKey;
-
     LayerType mRasterType;
 
     QgsRasterPipe mPipe;
 
     //! To save computations and possible infinite cycle of notifications
     QgsRectangle mLastRectangleUsedByRefreshContrastEnhancementIfNeeded;
+
+    QDomDocument mOriginalStyleDocument;
+    QDomElement mOriginalStyleElement;
 };
 
 // clazy:excludeall=qstring-allocations
