@@ -21,12 +21,8 @@ __author__ = 'Victor Olaya'
 __date__ = 'August 2012'
 __copyright__ = '(C) 2012, Victor Olaya'
 
-# This will get replaced with a git SHA1 when you do a git archive
-
-__revision__ = '0c5774c0685d4b6f3e92a086234455708d1cd427'
-
 from qgis.PyQt.QtCore import QCoreApplication
-from qgis.core import QgsApplication, QgsProcessingModelAlgorithm, QgsMessageLog
+from qgis.core import QgsApplication, QgsProcessingAlgorithm
 from processing.gui.ContextAction import ContextAction
 from processing.modeler.ModelerDialog import ModelerDialog
 from qgis.core import Qgis
@@ -40,7 +36,7 @@ class EditModelAction(ContextAction):
         self.name = QCoreApplication.translate('EditModelAction', 'Edit Model…')
 
     def isEnabled(self):
-        return isinstance(self.itemData, QgsProcessingModelAlgorithm)
+        return isinstance(self.itemData, QgsProcessingAlgorithm) and self.itemData.provider().id() == "model"
 
     def execute(self):
         alg = self.itemData
